(ns com.aurellem.gb.species
  (:use (com.aurellem.gb assembly characters gb-driver util
                         constants))
  (:import [com.aurellem.gb.gb_driver SaveState]))



(def species-code->species-name
  {0x01  :rhydon
   0x02  :kangaskhan
   0x03  :nidoran♂
   0x04  :clefairy
   0x05  :spearow
   0x06  :voltorb
   0x07  :nidoking
   0x08  :slowbro
   0x09  :ivysaur
   0x0A  :exeggutor
   0x0B  :lickitung
   0x0C  :exeggcute
   0x0D  :grimer
   0x0E  :gengar
   0x0F  :nidoran♀
   0x10  :nidoqueen
   0x11  :cubone
   0x12  :rhyhorn
   0x13  :lapras
   0x14  :arcanine
   0x15  :mew
   0x16  :gyarados
   0x17  :shellder
   0x18  :tentacool
   0x19  :gastly
   0x1A  :scyther
   0x1B  :staryu
   0x1C  :blastoise
   0x1D  :pinsir
   0x1E  :tangela
   0x21  :growlithe
   0x22  :onix
   0x23  :fearow
   0x24  :pidgey
   0x25  :slowpoke
   0x26  :kadabra
   0x27  :graveler
   0x28  :chansey
   0x29  :machoke
   0x2A  :mr-mime
   0x2B  :hitmonlee
   0x2C  :hitmonchan
   0x2D  :arbok
   0x2E  :parasect
   0x2F  :psyduck
   0x30  :drowzee
   0x31  :golem
   0x33  :magmar
   0x35  :electabuzz
   0x36  :magneton
   0x37  :koffing
   0x39  :mankey
   0x3A  :seel
   0x3B  :diglett
   0x3C  :tauros
   0x40  :farfetch
   0x41  :venonat
   0x42  :dragonite
   0x46  :doduo
   0x47  :poliwag
   0x48  :jynx
   0x49  :moltres
   0x4A  :articuno
   0x4B  :zapdos
   0x4C  :ditto
   0x4D  :meowth
   0x4E  :krabby
   0x52  :vulpix
   0x53  :ninetales
   0x54  :pikachu
   0x55  :raichu
   0x58  :dratini
   0x59  :dragonair
   0x5A  :kabuto
   0x5B  :kabutops
   0x5C  :horsea
   0x5D  :seadra
   0x60  :sandshrew
   0x61  :sandslash
   0x62  :omanyte
   0x63  :omastar
   0x64  :jigglypuff
   0x65  :wigglytuff
   0x66  :eevee
   0x67  :flareon
   0x68  :jolteon
   0x69  :vaporeon
   0x6A  :machop
   0x6B  :zubat
   0x6C  :ekans
   0x6D  :paras
   0x6E  :poliwhirl
   0x6F  :poliwrath
   0x70  :weedle
   0x71  :kakuna
   0x72  :beedrill
   0x74  :dodrio
   0x75  :primeape
   0x76  :dugtrio
   0x77  :venomoth
   0x78  :dewgong
   0x7B  :caterpie
   0x7C  :metapod
   0x7D  :butterfree
   0x7E  :machamp
   0x80  :golduck
   0x81  :hypno
   0x82  :golbat
   0x83  :mewtwo
   0x84  :snorlax
   0x85  :magikarp
   0x88  :muk
   0x8A  :kingler
   0x8B  :cloyster
   0x8D  :electrode
   0x8E  :clefable
   0x8F  :weezing
   0x90  :persian
   0x91  :marowak
   0x93  :haunter
   0x94  :abra
   0x95  :alakazam
   0x96  :pidgeotto
   0x97  :pidgeot
   0x98  :starmie
   0x99  :bulbasaur
   0x9A  :venusaur
   0x9B  :tentacruel
   0x9D  :goldeen
   0x9E  :seaking
   0xA3  :ponyta
   0xA4  :rapidash
   0xA5  :rattata
   0xA6  :raticate
   0xA7  :nidorino
   0xA8  :nidorina
   0xA9  :geodude
   0xAA  :porygon
   0xAB  :aerodactyl
   0xAD  :magnemite
   0xB0  :charmander
   0xB1  :squirtle
   0xB2  :charmeleon
   0xB3  :wartortle
   0xB4  :charizard
   0xB9  :oddish
   0xBA  :gloom
   0xBB  :vileplume
   0xBC  :bellsprout
   0xBD  :weepinbell
   0xBE  :victreebel})

(def species-ID-1 0xD163)

(def species-name->species-code
  (zipmap (vals species-code->species-name)
          (keys species-code->species-name)))

(defn pokemon-species-data-address [poke-num]
  (+ species-ID-1 poke-num))

(defn read-species
  ([^SaveState state poke-num]
     (assert (<= 0 poke-num 5))
     (species-code->species-name
      (aget (memory state)
            (pokemon-species-data-address poke-num))))
  ([poke-num]
     (read-species @current-state poke-num)))

(defn set-species
  ([^SaveState state poke-num species-name]
     (assert (<= 0 poke-num 5))
     (let [species-code (species-name->species-code
                         species-name
                         species-name)]
       
       (set-memory state (pokemon-species-data-address poke-num)
                   species-code)))
  ([poke-num species-name]
     (set-species @current-state poke-num species-name)))


(def pokemon-1-species2-address 0xD16A)

(defn pokemon-species2-address [poke-num]
  (+ pokemon-1-species2-address
     (* pokemon-record-width poke-num)))

(defn read-species2
  ([^SaveState state poke-num]
     (species-code->species-name
      (aget (memory state)
            (pokemon-species2-address poke-num))))
  ([poke-num]
     (read-species2 @current-state poke-num)))

(defn set-species2
  ([^SaveState state poke-num new-species]
     (let [species-code (species-name->species-code
                         new-species
                         new-species)]
       
       (set-memory state (pokemon-species2-address poke-num)
                   species-code)))
  ([poke-num new-species]
     (set-species2 @current-state poke-num new-species)))


(def pokedex-num->species-name
  {1       :bulbasaur     
   2       :ivysaur       
   3       :venusaur      
   4       :charmander    
   5       :charmeleon    
   6       :charizard     
   7       :squirtle      
   8       :wartortle     
   9       :blastoise     
   10      :caterpie      
   11      :metapod       
   12      :butterfree    
   13      :weedle        
   14      :kakuna        
   15      :beedrill      
   16      :pidgey        
   17      :pidgeotto     
   18      :pidgeot       
   19      :rattata       
   20      :raticate      
   21      :spearow       
   22      :fearow        
   23      :ekans         
   24      :arbok         
   25      :pikachu       
   26      :raichu        
   27      :sandshrew     
   28      :sandslash     
   29      :nidoran♀      
   30      :nidorina      
   31      :nidoqueen     
   32      :nidoran♂      
   33      :nidorino      
   34      :nidoking      
   35      :clefairy      
   36      :clefable      
   37      :vulpix        
   38      :ninetales     
   39      :jigglypuff    
   40      :wigglytuff    
   41      :zubat         
   42      :golbat        
   43      :oddish        
   44      :gloom         
   45      :vileplume     
   46      :paras         
   47      :parasect      
   48      :venonat       
   49      :venomoth      
   50      :diglett       
   51      :dugtrio       
   52      :meowth        
   53      :persian       
   54      :psyduck       
   55      :golduck       
   56      :mankey        
   57      :primeape      
   58      :growlithe     
   59      :arcanine      
   60      :poliwag       
   61      :poliwhirl     
   62      :poliwrath     
   63      :abra          
   64      :kadabra       
   65      :alakazam      
   66      :machop        
   67      :machoke       
   68      :machamp       
   69      :bellsprout    
   70      :weepinbell    
   71      :victreebel    
   72      :tentacool     
   73      :tentacruel    
   74      :geodude       
   75      :graveler      
   76      :golem         
   77      :ponyta        
   78      :rapidash      
   79      :slowpoke      
   80      :slowbro       
   81      :magnemite     
   82      :magneton      
   83      :farfetch    
   84      :doduo         
   85      :dodrio        
   86      :seel          
   87      :dewgong       
   88      :grimer        
   89      :muk           
   90      :shellder      
   91      :cloyster      
   92      :gastly        
   93      :haunter       
   94      :gengar        
   95      :onix          
   96      :drowzee       
   97      :hypno         
   98      :krabby        
   99      :kingler       
   100     :voltorb       
   101     :electrode     
   102     :exeggcute     
   103     :exeggutor     
   104     :cubone        
   105     :marowak       
   106     :hitmonlee     
   107     :hitmonchan    
   108     :lickitung     
   109     :koffing       
   110     :weezing       
   111     :rhyhorn       
   112     :rhydon        
   113     :chansey       
   114     :tangela       
   115     :kangaskhan    
   116     :horsea        
   117     :seadra        
   118     :goldeen       
   119     :seaking       
   120     :staryu        
   121     :starmie       
   122     :mr-mime      
   123     :scyther       
   124     :jynx          
   125     :electabuzz    
   126     :magmar        
   127     :pinsir        
   128     :tauros        
   129     :magikarp      
   130     :gyarados      
   131     :lapras        
   132     :ditto         
   133     :eevee         
   134     :vaporeon      
   135     :jolteon       
   136     :flareon       
   137     :porygon       
   138     :omanyte       
   139     :omastar       
   140     :kabuto        
   141     :kabutops      
   142     :aerodactyl    
   143     :snorlax       
   144     :articuno      
   145     :zapdos        
   146     :moltres       
   147     :dratini       
   148     :dragonair     
   149     :dragonite     
   150     :mewtwo         
   151     :mew})
         
(def species-name->pokedex-num
  (zipmap (vals pokedex-num->species-name)
          (keys pokedex-num->species-name)))
